/*******************************************************************
 *
 *	File:		SCapEng.cpp
 *
 *	Author:		Peter van Sebille (peter@yipton.net)
 *
 *	(c) Copyright 2003, Peter van Sebille
 *	All Rights Reserved
 *
 *******************************************************************/

#include "SCapEng.h"
#include <e32keys.h>
#include <quartzkeys.h>
#include <eikenv.h>

const TInt KOptionFileVersion = 1;

static const TInt sCaptureKeyCodes[] = 
{
	EKeyApplication1,		// ECaptureCamera
	EKeyApplication0,		// ECaptureBrowser
	EKeyDevice0,			// ECapturePower
	EQuartzKeyConfirm,		// ECaptureJogdialSelect
};


CSCapEngine::~CSCapEngine()
{
	StopCapture();
	delete iMdaUtil;
}

CSCapEngine* CSCapEngine::NewL()
{
	CSCapEngine*	self = new(ELeave)CSCapEngine;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(self);

	return self;
}


void CSCapEngine::ConstructL()
{
	iMdaUtil = CMdaImageBitmapToFileUtility::NewL(*this);
	iJfifFormat.iSettings.iSampleScheme = TMdaJpgSettings::EColor444;
	
	CEikonEnv::Static()->FsSession().MkDirAll(_L("c:\\system\\apps\\scap\\"));
	DefaultOptions();
	LoadOptions();
}


void CSCapEngine::SaveOptions(const TSCapOptions& aSCapOptions)
{
	RFile	file;
	TInt err = file.Replace(CEikonEnv::Static()->FsSession(), _L("c:\\system\\apps\\scap\\scap.ini"), EFileRead);
	if (err != KErrNone)
		err = file.Create(CEikonEnv::Static()->FsSession(), _L("c:\\system\\apps\\scap\\scap.ini"), EFileRead);
	if (err == KErrNone)
	{
		TInt	version = KOptionFileVersion;
		TPckg<TInt>	versionPkg(version);
		err = file.Write(versionPkg);
		if (err == KErrNone)
		{
			TPckg<TSCapOptions>	optionsPkg(aSCapOptions);
			err = file.Write(optionsPkg);
			file.Close();
		}
	}

	if (err == KErrNone)
	{
		iSCapOptions = aSCapOptions;
		User::InfoPrint(_L("Settings saved"));
	}
	else
		User::InfoPrint(_L("Could not save settings"));
}


void CSCapEngine::DefaultOptions()
{
	iSCapOptions.iKey = ECaptureCamera;
	iSCapOptions.iManual = ETrue;
	iSCapOptions.iNoVisual = EFalse;
	iSCapOptions.iSCapAudioOption = EAudioDefault;
	iSCapOptions.iSCapImageOptions.iBaseFileName = _L("screen");
	iSCapOptions.iSCapImageOptions.iImageFormat = EFormatJpg;
	iSCapOptions.iSCapImageOptions.iQualityFactor = 90;
	iSCapOptions.iSCapImageOptions.iStorageInternal = ETrue;
}


void CSCapEngine::LoadOptions()
{
	RFile	file;
	TInt err = file.Open(CEikonEnv::Static()->FsSession(), _L("c:\\system\\apps\\scap\\scap.ini"), EFileRead);
	if (err == KErrNone)
	{
		TInt	version;
		TPckg<TInt>	versionPkg(version);
		err = file.Read(versionPkg);
		if (err == KErrNone)
		{
			if (version == 1)
			{
				TPckg<TSCapOptions>	optionsPkg(iSCapOptions);
				err = file.Read(optionsPkg);
			}
		}
		file.Close();
	}
}


void CSCapEngine::StartStopCapture()
{
	if (iIsCapturing)
		StopCapture();
	else
		StartCapture();
}

void CSCapEngine::DestroyBitmap()
{
	delete iBitmap;
	iBitmap = NULL;
}

void CSCapEngine::CreateBitmapL()
{
	TSize	screenSize = CEikonEnv::Static()->ScreenDevice()->SizeInPixels();
	TDisplayMode displayMode = CEikonEnv::Static()->ScreenDevice()->DisplayMode();

	iBitmap = new(ELeave)CFbsBitmap;
	User::LeaveIfError(iBitmap->Create(screenSize, displayMode));
}

void CSCapEngine::StartCaptureL()
{
	DestroyBitmap();
	CreateBitmapL();

	iIsCapturing = ETrue;
	CaptureKey();
}

void CSCapEngine::StartCapture()
{
	TRAPD(err, StartCaptureL())
	if (err)
		StopCapture();
}


void CSCapEngine::StopCapture()
{
	iIsCapturing = EFalse;
	CancelCaptureKey();
	DestroyBitmap();
}


void CSCapEngine::CaptureScreen()
{
	TInt err = CEikonEnv::Static()->ScreenDevice()->CopyScreenToBitmap(iBitmap);
	if (err == KErrNone)
		SaveImage();
	else
		SavingImageComplete(err);
}


void CSCapEngine::CaptureKey()
{
	iCaptureKey = CEikonEnv::Static()->RootWin().CaptureKey(sCaptureKeyCodes[iSCapOptions.iKey], 0, 0, 2);
}

void CSCapEngine::CancelCaptureKey()
{
	if (iCaptureKey)
		CEikonEnv::Static()->RootWin().CancelCaptureKey(iCaptureKey);
	iCaptureKey = 0;
}


void CSCapEngine::CaptureScreenIfCaptureKey(TInt aKeyCode)
{
	if (iIsCapturing && (aKeyCode == sCaptureKeyCodes[iSCapOptions.iKey]))
	{
		if (!IsBusy())
			CaptureScreen();
		else
			User::InfoPrint(_L("Screen capture busy"));
	}
}


void CSCapEngine::SaveImage()
{
	if (iSaveImageObserver)
		iSaveImageObserver->StartedSavingImage();
	
	TRAPD(err, SaveImageL());
	if (err)
		SavingImageComplete(err);
}


void CSCapEngine::MkFileNameL(TFileName& aFileName, const TDesC& aExt)
{
	TInt	err = KErrNone;
	TBool internal = iSCapOptions.iSCapImageOptions.iStorageInternal;
	if (!MkFileName(aFileName, aExt, internal))
	{
		internal = internal ? EFalse : ETrue;
		if (!MkFileName(aFileName, aExt, internal))
			err = KErrOverflow;
	}
	User::LeaveIfError(err);
}

TBool CSCapEngine::MkFileName(TFileName& aFileName, const TDesC& aExt, TBool aInternal)
{
	TBuf<64>	imagePath;
	RFs&	fs = CEikonEnv::Static()->FsSession();

	if (aInternal)
		imagePath = _L("c:\\Documents\\Media Files\\Image\\scap\\");
	else
		imagePath = _L("d:\\Media Files\\Image\\scap\\");
	fs.MkDirAll(imagePath);

	TInt num = 0;
	do
	{
		TUint	dummyAttrib;
		if (num == 0)
			aFileName.Format(_L("%S%S%S"), &imagePath, &iSCapOptions.iSCapImageOptions.iBaseFileName , &aExt);
		else 
			aFileName.Format(_L("%S%S-%d%S"), &imagePath, &iSCapOptions.iSCapImageOptions.iBaseFileName, num, &aExt);

		if (fs.Att(aFileName, dummyAttrib) != KErrNone)
			break;
		else
			num++;
	} while (num < 100);

	return (num < 100) ? ETrue : EFalse;
}

void CSCapEngine::SaveImageL()
{
	TFileName	fileName;
	TBuf<10>	ext;
	
	
	TMdaClipFormat*	format = NULL;
	TMdaPackage*	codec = NULL;

	switch (iSCapOptions.iSCapImageOptions.iImageFormat)
	{
		case EFormatBmp24:
			format	= &iBmpFormat;
			codec	= &iBmp24BppCodec;
			ext = _L(".bmp");
			break;

		case EFormatBmp8:
			format	= &iBmpFormat;
			codec	= &iBmp8BppCodec;
			ext = _L(".bmp");
			break;

		case EFormatGif:
			format = &iGifFormat;
			codec = &iGifCodec;
			ext = _L(".gif");
			break;
	
		case EFormatJpg:
			iJfifFormat.iSettings.iQualityFactor = iSCapOptions.iSCapImageOptions.iQualityFactor;
			format = &iJfifFormat;
			ext = _L(".jpg");
			break;

		default:
			User::Leave(KErrNotSupported);
			break;
	}

	MkFileNameL(fileName, ext);
	iMdaUtil->CreateL(fileName, format, codec, NULL);
}


void CSCapEngine::MiuoCreateComplete(TInt aError)
{
	if (aError == KErrNone)
	{
		TRAPD(err, iMdaUtil->ConvertL(*iBitmap, 0));
		if (err)
			SavingImageComplete(err);
	}
}

void CSCapEngine::MiuoConvertComplete(TInt aError)
{
	SavingImageComplete(aError);
}


void CSCapEngine::MiuoOpenComplete(TInt aError)
{
	if (aError)
		User::InfoPrint(_L("MiuoOpenComplete failed"));
}


void CSCapEngine::SavingImageComplete(TInt aErr)
{
	if (iSaveImageObserver)
		iSaveImageObserver->FinishedSavingImage();

	if (aErr)
	{
		User::InfoPrint(_L("Capture failed"));
	}
	else
	{
		if (!iSCapOptions.iNoVisual)
			User::InfoPrint(_L("Screen captured"));
		if (iSCapOptions.iSCapAudioOption != EAudioSilent)
			PlayOkSound();
	}
	iIsSavingImage = EFalse;
	StopCapture();
	if (iSCapOptions.iManual)
		StartCapture();
}


#ifdef __WINSCW__
_LIT(KSCapDefaultFileName, "z:\\system\\data\\QSystemSounds\\QScreenClick.wav");
#else
_LIT(KSCapDefaultFileName, "z:\\system\\data\\QSystemSounds\\ding.wav");
#endif

void CSCapEngine::PlayOkSound()
{
	delete iUtility;
	iUtility = NULL;
	TPtrC	ptr(KSCapDefaultFileName);
	if (iSCapOptions.iSCapAudioOption == EAudioUserDefined)
		ptr.Set(iSCapOptions.iAudioClip);
	TRAPD(err, iUtility = CMdaAudioPlayerUtility::NewFilePlayerL(ptr, *this));
	if (err)
		MapcPlayComplete(err);
}

void CSCapEngine::MapcInitComplete(TInt aError, const TTimeIntervalMicroSeconds& /*aDuration*/)
{
	if (aError)
		MapcPlayComplete(aError);
	else
		iUtility->Play();
}

void CSCapEngine::MapcPlayComplete(TInt /*aError*/)
{
	iIsPlaying = EFalse;
	delete iUtility;
	iUtility = NULL;
}

TBool CSCapEngine::IsBusy()
{
	return (/*iIsPlaying || */ iIsSavingImage);
}
